local AddonName, SM = ...

-- Bill's Utils
local ChkBox = BillsUtils.ChkBox
local CheckClick = BillsUtils.CheckClick
local SliderMW = BillsUtils.SliderMW

BillsUtils.Locals[#BillsUtils.Locals +1] = function ()
	ChkBox = BillsUtils.ChkBox
	CheckClick = BillsUtils.CheckClick
	SliderMW = BillsUtils.SliderMW
end

-- ****************************************************************************
-- * Color Changing Button Stuff                                              *
-- ****************************************************************************

local ButtonTextures = {
	RED = {
		up = "Interface\\Buttons\\UI-Panel-Button-Up",
		down = "Interface\\Buttons\\UI-Panel-Button-Down",
		highlight = "Interface\\Buttons\\UI-Panel-Button-Highlight",
	},
	GREEN = {
		up = "Interface\\AddOns\\SlashMagic\\Green-Panel-Button-Up",
		down = "Interface\\AddOns\\SlashMagic\\Green-Panel-Button-Down",
		highlight = "Interface\\AddOns\\SlashMagic\\Green-Panel-Button-Highlight",
	},
	BLUE = {
		up = "Interface\\AddOns\\SlashMagic\\Blue-Panel-Button-Up",
		down = "Interface\\AddOns\\SlashMagic\\Blue-Panel-Button-Down",
		highlight = "Interface\\AddOns\\SlashMagic\\Blue-Panel-Button-Highlight",
	},
	GREY = {
		up = "Interface\\AddOns\\SlashMagic\\Grey-Panel-Button-Up",
		down = "Interface\\AddOns\\SlashMagic\\Grey-Panel-Button-Down",
		highlight = "Interface\\AddOns\\SlashMagic\\Grey-Panel-Button-Highlight",
	},
}

local function GetSkin(self)
	return self.__skin
end

local function SetSkin(self, skin)
	if self.__skin == skin then return end
	self.__skin = skin

	if self:IsMouseOver() and IsMouseButtonDown() then
		local down = ButtonTextures[skin].down
		self.Left:SetTexture(down)
		self.Right:SetTexture(down)
		self.Middle:SetTexture(down)
	else
		local up = ButtonTextures[skin].up
		self.Left:SetTexture(up)
		self.Right:SetTexture(up)
		self.Middle:SetTexture(up)
	end

	self:SetHighlightTexture(ButtonTextures[skin].highlight, "ADD")
	self:GetHighlightTexture():SetTexCoord(0, 80/128, 0, 22/32)
end

local function OnMouseDown(self, button)
	local down = ButtonTextures[self.__skin].down
	self.Left:SetTexture(down)
	self.Right:SetTexture(down)
	self.Middle:SetTexture(down)
end

local function OnMouseUp(self, button)
	local up = ButtonTextures[self.__skin].up
	self.Left:SetTexture(up)
	self.Right:SetTexture(up)
	self.Middle:SetTexture(up)
end

local function NewButton(parent, skin, name)
	local button = CreateFrame("Button", name, parent or UIParent)
	button:SetSize(40, 22)

	local left = button:CreateTexture(nil, "BACKGROUND")
	left:SetPoint("TOPLEFT")
	left:SetPoint("BOTTOMLEFT")
	left:SetWidth(12)
	left:SetTexCoord(0, 12/128, 0, 22/32)
	button.Left = left

	local right = button:CreateTexture(nil, "BACKGROUND")
	right:SetPoint("TOPRIGHT")
	right:SetPoint("BOTTOMRIGHT")
	right:SetWidth(12)
	right:SetTexCoord(68/128, 80/128, 0, 22/32)
	button.Right = right

	local middle = button:CreateTexture(nil, "BACKGROUND")
	middle:SetPoint("TOPLEFT", left, "TOPRIGHT")
	middle:SetPoint("BOTTOMRIGHT", right, "BOTTOMLEFT")
	middle:SetTexCoord(12/128, 68/128, 0, 22/32)
	button.Middle = middle

	local text = button:CreateFontString(nil, "OVERLAY", "GameFontNormal")
	text:SetAllPoints(true)
	text:SetJustifyH("CENTER")
	text:SetJustifyV("CENTER")
	button:SetFontString(text)
	button:SetNormalFontObject("GameFontNormal")
	button:SetHighlightFontObject("GameFontHighlight")	

	button:SetScript("OnMouseDown", OnMouseDown)
	button:SetScript("OnMouseUp", OnMouseUp)
	button:SetScript("OnShow", OnMouseUp)

	button.GetSkin = GetSkin
	button.SetSkin = SetSkin

	SetSkin(button, skin or "RED")

	return button
end

-- ****************************************************************************
-- * Frame begins here                                                        *
-- ****************************************************************************

SM.ConfigFrame = CreateFrame("Frame" , "SMConfigFrame", UIParent )
local me = SM.ConfigFrame
me:SetFrameStrata("DIALOG")
me:Hide()

local CmdListButtons = {}
local DisplayedButtons = 0
local Selected = false
local OldSelected = false
local BottomButtons = { "Active", "New", "Delete", "Default", "Restore", "Reset", "Save", "Close" }

local Backdrop={
	bgFile = "Interface\\DialogFrame\\UI-DialogBox-Background",
	edgeFile="Interface\\DialogFrame\\UI-DialogBox-Border",
	tile = true,
	tileSize = 16,
	edgeSize = 16,
	insets = {
		left = 0,
		right = 0,
		top = 0,
		bottom = 0
	}
}
local Backdrop2={
	bgFile = "",
	edgeFile="Interface\\DialogFrame\\UI-DialogBox-Border",
	tile = true,
	tileSize = 16,
	edgeSize = 16,
	insets = {
		left = 0,
		right = 0,
		top = 0,
		bottom = 0
	}
}

	
me:EnableMouse(true)
me:SetPoint("CENTER", UIParent, "CENTER")
me:SetHeight( .80 * GetScreenHeight() )
me:SetWidth( .80 * GetScreenWidth() )
me:SetBackdrop( Backdrop )
me:SetBackdropColor( 128, 128,128, 1)
me:SetBackdropBorderColor( 255, 255, 255, 1)
me:SetMovable(true)
me:SetUserPlaced(false)
me:SetClampedToScreen(true)

local CmdListDisplayUpdate = function()
	local list = {}
	local offset
	local size = 0
	local cmds = SlashMagic_Commands.cmds
	local code = SlashMagic_Commands.code
	local desc = SlashMagic_Commands.desc
	local dcmds = SM.Commands_Defaults.cmds
	
	for index in pairs( cmds ) do
		list[#list + 1] = index
	end
	table.sort( list )
	size = #list
	if not(Selected) then
		Selected = list[1]
	end
	
	if size <= DisplayedButtons then
		me.BFslider:SetValue(0)
		me.BFslider:Disable()
	else
		me.BFslider:Enable()
		me.BFslider:SetMinMaxValues( 0, size - DisplayedButtons)
	end
	
	offset = me.BFslider:GetValue()
	
	for x = 1, DisplayedButtons do
		local which = x + offset
		local button = CmdListButtons[ x ]
		if which <= size then
			button:SetText(list[which])
			if Selected == list[which] then
				button:SetSkin("BLUE")
				if cmds[list[which]] then
					me.Active:SetText("Active")
					me.Active:SetSkin("GREEN")
				else
					me.Active:SetText("Not Active")
					me.Active:SetSkin("RED")
				end
			elseif cmds[ list[which] ] then
				button:SetSkin("GREEN")
			else
				button:SetSkin("RED")
			end
		else
			button:SetText("")
			button:SetSkin("GREY")
		end
	end
	if type(dcmds[Selected]) == "boolean" then
		me.Default:SetSkin("GREEN")
	else
		me.Default:SetSkin("GREY")
	end
	
	if OldSelected ~= Selected then
		OldSelected = Selected
		me.NameFrameText:SetText( ("Name: %s"):format(Selected) )
		local whatisit = type(SlashMagic_Commands.vers[Selected])
		if whatisit == "number" then
			me.VersionText:SetText(("Version: %.2f"):format(SlashMagic_Commands.vers[Selected]))
		elseif whatisit == "string" then
			me.VersionText:SetText(("Version: %s"):format(SlashMagic_Commands.vers[Selected]))
		else
			me.VersionText:SetText(" ")
		end
		me.DescFrameEBox:SetText( SlashMagic_Commands.desc[Selected] )
		me.CodeFrameEBox:SetText( string.join("\013",unpack(SlashMagic_Commands.code[Selected])))
	end
end

StaticPopupDialogs["SLASHMAGIC_DELETECOMMAND"] = {
	text = "Do you want to delete the selected command %s ?",
	button1 = "Yes",
	button2 = "No",
	OnAccept = function()
		SM.Disable(Selected)
		SlashMagic_Commands.cmds[Selected] = nil
		SlashMagic_Commands.code[Selected] = nil
		SlashMagic_Commands.desc[Selected] = nil
		Selected = false
		CmdListDisplayUpdate()
	end,
	timeout = 0,
	whileDead = true,
	hideOnEscape = true,
	preferredIndex = 3,  -- avoid some UI taint, see http://www.wowace.com/announcements/how-to-avoid-some-ui-taint/ 
}
StaticPopupDialogs["SLASHMAGIC_RESETCOMMAND"] = {
	text = "Are you want to reset the addon back to \"Factory\" conditions. This will remove all custom created commands and restore the original functionality to the addon.",
	button1 = "Yes",
	button2 = "No",
	OnAccept = function()
		SM.Disable(Selected)
		SM.Reset()
		Selected = false
		CmdListDisplayUpdate()
	end,
	timeout = 0,
	whileDead = true,
	hideOnEscape = true,
	preferredIndex = 3,  -- avoid some UI taint, see http://www.wowace.com/announcements/how-to-avoid-some-ui-taint/ 
}

StaticPopupDialogs["SLASHMAGIC_DEFAULTCOMMAND"] = {
	text = "Do you want to restore this command back to \"Factory\" default?",
	button1 = "Yes",
	button2 = "No",
	OnAccept = function()
		SM.Reset1( Selected )
		OldSelected = false
		CmdListDisplayUpdate()
	end,
	timeout = 0,
	whileDead = true,
	hideOnEscape = true,
	preferredIndex = 3,  -- avoid some UI taint, see http://www.wowace.com/announcements/how-to-avoid-some-ui-taint/ 
}

StaticPopupDialogs["SLASHMAGIC_RESTORECOMMAND"] = {
	text = "Are you sure you want to discard all of your changes since the last save?",
	button1 = "Yes",
	button2 = "No",
	OnAccept = function()
		OldSelected = false
		CmdListDisplayUpdate()
	end,
	timeout = 0,
	whileDead = true,
	hideOnEscape = true,
	preferredIndex = 3,  -- avoid some UI taint, see http://www.wowace.com/announcements/how-to-avoid-some-ui-taint/ 
}



me:SetScript("OnEvent",function(self, event, ...)
	if event == "ADDON_LOADED" then
		local arg = ...
		if arg == AddonName then
			--me:Show()
		end
	end
end)

me:SetScript("OnShow", function(self)
	if DisplayedButtons == 0 then
		me:SetHeight( .80 * GetScreenHeight() )
		me:SetWidth( .80 * GetScreenWidth() )
		
		me.Enable:SetChecked(SlashMagic_Settings.Enable)
		
		local MaxButtons = 0
		local BHeight = 0
		local BFHeight = me.ButtonFrame:GetHeight() / me.ButtonFrame:GetEffectiveScale()
		local whatsleft
		local PrevButton
		
		repeat
			DisplayedButtons = DisplayedButtons + 1
			CmdListButtons[DisplayedButtons] = NewButton( me.ButtonFrame, "RED", "SMCmdListButton"..DisplayedButtons )
			local Button = CmdListButtons[DisplayedButtons]
			if DisplayedButtons == 1 then
				Button:SetPoint("TOPLEFT", me.ButtonFrame,  "TOPLEFT", 5, -5)
				BHeight = Button:GetHeight() * .89 
				MaxButtons = math.floor(BFHeight / BHeight)
				whatsleft = BFHeight % BHeight
			else
				Button:SetPoint("TOP", PrevButton,  "BOTTOM" )
			end
			Button:SetWidth( 140 )
			Button:SetScript("OnClick", function(self, button, down)
				if button ~= "LeftButton" or self:GetSkin() == "GREY" then
					return
				end
				PlaySound( "GAMEGENERICBUTTONPRESS" )
				local text = self:GetText()
				if text ~= "" and text ~= nil then
					Selected = text
					CmdListDisplayUpdate()
				end
				return
			end)
			Button:SetScript("OnEnter", function(self)
				me.HintFrameText:SetText( "Select the command you would like to edit, or scroll the list with your mousewheel. (Holding shift will scroll 5x faster)" )
			end)
			Button:SetScript("OnLeave", function(self)
				me.HintFrameText:SetText( " " )
			end)
			PrevButton = Button
		until DisplayedButtons == MaxButtons
		me:SetHeight( me:GetHeight() - whatsleft + 7)
		CmdListDisplayUpdate()
		me.BFslider:SetValue(0)
		
		local BottomButtonWidth = me.HintFrame:GetWidth() / (#BottomButtons)
		
		for x = 1, #BottomButtons do
			me[BottomButtons[x]]:SetWidth(BottomButtonWidth)
		end
	end
end)

-- Title region
me.TitleRegion = me:CreateTitleRegion()
me.TitleRegion:SetSize( 324, 64)
me.TitleRegion:SetPoint( "TOP", me, "TOP", 0,12)

-- Title frame header
me.TitleBkgnd=me:CreateTexture(nil , "ARTWORK")
me.TitleBkgnd:SetTexture("Interface\\DialogFrame\\UI-DialogBox-Header")
me.TitleBkgnd:SetSize( 324, 64)
me.TitleBkgnd:SetPoint( "TOP", me, "TOP", 0,12)

-- Title frame text
me.Title = me:CreateFontString( nil , "OVERLAY", "GameFontHighlight")
me.Title:SetText("Slash Magic")
me.Title:SetPoint( "CENTER", me.TitleBkgnd, "CENTER", 0, 12 )
me.Title:SetJustifyH( "CENTER" )
me.Title:SetJustifyV( "MIDDLE" )

me.Enable = ChkBox( nil, "SMConfig", "Enable", me, CheckClick, "Enables the addon to function", nil, 15, -30 )
me.Enable:SetScript("OnEnter", function(self)
	me.HintFrameText:SetText( "This will enable/disable the addon. Slash commands will not work when the addon is disabled but you can edit them. This setting takes effect immediately." )
end)
me.Enable:SetScript("OnLeave", function(self)
	me.HintFrameText:SetText( " " )
end)
me.Enable:SetScript("OnClick", function(self)
	SlashMagic_Settings.Enable = self:GetChecked() == 1
	if SlashMagic_Settings.Enable then
		SM.AddonEnable()
	else
		SM.AddonDisable()
	end
end)

me.ButtonFrame = CreateFrame("Frame" , "SMCommandButtonFrame", me )
local ButtonFrame = me.ButtonFrame
ButtonFrame:SetBackdrop( Backdrop2 )
ButtonFrame:SetBackdropColor( 128, 128,128, 1)
ButtonFrame:SetBackdropBorderColor( 255, 255, 255, 1)
ButtonFrame:SetPoint( "TOPLEFT", me, "TOPLEFT", 10, -70 )
ButtonFrame:SetPoint( "BOTTOMLEFT", me, "BOTTOMLEFT", 10, 92 )
ButtonFrame:SetWidth(150)
ButtonFrame:EnableMouseWheel(1)

ButtonFrame:SetScript("OnMouseWheel", function(self, delta)
	SliderMW(me.BFslider, -delta)
end)

ButtonFrame:SetScript("OnEnter", function(self)
	me.HintFrameText:SetText( "Select the command you would like to edit, or scroll the list with your mousewheel. (Holding shift will scroll 5x faster)" )
end)
ButtonFrame:SetScript("OnLeave", function(self)
	me.HintFrameText:SetText( " " )
end)

me.BFslider = CreateFrame( "Slider", "SMBFSlider", me, "OptionsSliderTemplate" )
local BFslider = me.BFslider
BFslider:SetMinMaxValues( 0, 1 )
BFslider:SetOrientation("VERTICAL")
BFslider:SetPoint("TOPLEFT", me.ButtonFrame ,"TOPRIGHT", -4, 0)
BFslider:SetPoint("BOTTOM", me.ButtonFrame ,"BOTTOM" )
BFslider:SetValueStep( 1 )
BFslider:SetWidth( 16 )	
BFslider:SetScript( "OnValueChanged", function( self )
	CmdListDisplayUpdate()
end)
getglobal(BFslider:GetName() .. 'Low'):SetText( " " )
getglobal(BFslider:GetName() .. 'High'):SetText( " " )
BFslider:SetScript("OnMouseWheel", function(self, delta)
	SliderMW(self, -delta)
end)
BFslider:SetScript("OnEnter", function(self)
	me.HintFrameText:SetText( "Scroll the list with your mousewheel. (Holding shift will scroll 5x faster)" )
end)
BFslider:SetScript("OnLeave", function(self)
	me.HintFrameText:SetText( " " )
end)

-- BottomButtons = { "Active", "New", "Delete", "Default", "Restore", "Reset", "Save", "Close" }
local BBHelpText = {"This button enables / disables the slash command. Green is enabled, Red is disabled", -- Active
					"This button allows you to make a new slash command", -- New
					"This button deletes the selected slash command", -- Delete
					"This button will restore an edited default command, It will turn green when a default command is selected.", -- Default
					"This button will restore an edited command back to the state of its last save", -- Restore
					"This button will reset the addon back to a fresh install state", -- Reset
					"This button will save the above description/code for the selected command", -- Save
					"This button will close the config window without saving anything",
					}
for x = 1, #BottomButtons do
	me[BottomButtons[x]] = NewButton( me, "RED", "SM"..BottomButtons[x].."Button" )
	local Button = me[BottomButtons[x]]
	if x == 1 then
		Button:SetPoint("BOTTOM", me, "BOTTOM", 0, 10)
		me[BottomButtons[x]]:SetPoint("LEFT", me.ButtonFrame, "LEFT")
	else
		Button:SetPoint("LEFT", me[BottomButtons[x-1]], "RIGHT")
	end
	Button:SetSize( 150, 25)
	Button:SetText(BottomButtons[x])
	Button.number = x
	Button:SetScript("OnEnter", function(self)
		me.HintFrameText:SetText( BBHelpText[ self.number ] )
	end)
	Button:SetScript("OnLeave", function(self)
		me.HintFrameText:SetText( " " )
	end)
end

me.Active:SetScript("OnClick", function(self, button, down)
	if button ~= "LeftButton" then
		return
	end
	PlaySound( "GAMEGENERICBUTTONPRESS" )
	SlashMagic_Commands.cmds[Selected] = not( SlashMagic_Commands.cmds[Selected] )
	if SlashMagic_Commands.cmds[Selected] then
		SM.Enable( Selected )
	else
		SM.Disable( Selected )
	end
	CmdListDisplayUpdate()
end)

me.New:SetScript("OnClick", function(self, button, down)
	if button ~= "LeftButton" then
		return
	end
	PlaySound( "GAMEGENERICBUTTONPRESS" )
	me.NewCmdFrame:Show()
end)

me.Delete:SetScript("OnClick", function(self, button, down)
	if button ~= "LeftButton" then
		return
	end
	PlaySound( "GAMEGENERICBUTTONPRESS" )
	StaticPopup_Show("SLASHMAGIC_DELETECOMMAND", Selected)
end)

me.Default:SetScript("OnClick", function(self, button, down)
	if button ~= "LeftButton" then
		return
	end
	if self:GetSkin() == "GREY" then 
		return
	end
	PlaySound( "GAMEGENERICBUTTONPRESS" )
	StaticPopup_Show("SLASHMAGIC_DEFAULTCOMMAND", Selected)
end)

me.Restore:SetScript("OnClick", function(self, button, down)
	if button ~= "LeftButton" then
		return
	end
	PlaySound( "GAMEGENERICBUTTONPRESS" )
	--self:SetSkin("BLUE")
	StaticPopup_Show("SLASHMAGIC_RESTORECOMMAND" )
end)

me.Reset:SetScript("OnClick", function(self, button, down)
	if button ~= "LeftButton" then
		return
	end
	PlaySound( "GAMEGENERICBUTTONPRESS" )
	--self:SetSkin("BLUE")
	StaticPopup_Show("SLASHMAGIC_RESETCOMMAND" )
end)

me.Close:SetScript("OnClick", function(self, button, down)
	if button ~= "LeftButton" then
		return
	end
	PlaySound( "GAMEGENERICBUTTONPRESS" )
	me:Hide()
end)

me.Save:SetScript("OnClick", function(self, button, down)
	if button ~= "LeftButton" then
		return
	end
	PlaySound( "GAMEGENERICBUTTONPRESS" )
	
	SlashMagic_Commands.desc[Selected] = me.DescFrameEBox:GetText()
	SlashMagic_Commands.vers[Selected] = "Custom"
	SlashMagic_Commands.code[Selected] = {string.split("\013", me.CodeFrameEBox:GetText()) }
	SM.Disable(Selected)
	SM.Enable(Selected)
	OldSelected = false
	CmdListDisplayUpdate()
	
end)

me.NameFrame = CreateFrame("Frame" , "SMNameFrame", me )
local NameFrame = me.NameFrame
NameFrame:SetBackdrop( Backdrop2 )
NameFrame:SetBackdropColor( 128, 128,128, 1)
NameFrame:SetBackdropBorderColor( 255, 255, 255, 1)
NameFrame:SetPoint( "TOPLEFT", me.ButtonFrame, "TOPRIGHT", 15, 0 )
NameFrame:SetHeight( 30 )
NameFrame:SetPoint( "RIGHT", me, "RIGHT", -10, 0)

me.NameFrameText = me.NameFrame:CreateFontString( nil , "OVERLAY", "ChatFontNormal")
local NameFrameText = me.NameFrameText
NameFrameText:SetPoint( "TOPLEFT", NameFrame, "TOPLEFT", 8, -8 )
NameFrameText:SetPoint( "BOTTOMRIGHT", NameFrame, "BOTTOMRIGHT", -200, 0 )
NameFrameText:SetJustifyH( "LEFT" )
NameFrameText:SetJustifyV( "TOP" )

me.VersionText =  NameFrame:CreateFontString( nil , "OVERLAY", "ChatFontNormal")
local VersionText = me.VersionText
VersionText:SetPoint( "TOPLEFT", NameFrameText, "TOPRIGHT")
VersionText:SetPoint( "BOTTOMLEFT", NameFrameText, "BOTTOMRIGHT")
VersionText:SetPoint( "RIGHT", NameFrame, "RIGHT", -5, 0 )
VersionText:SetJustifyH( "RIGHT" )
VersionText:SetJustifyV( "TOP" )

me.DescFrame = CreateFrame("Frame" , "SMDescFrame", me )
me.DescFrame:SetBackdrop( Backdrop2 )
me.DescFrame:SetBackdropColor( 128, 128,128, 1)
me.DescFrame:SetBackdropBorderColor( 255, 255, 255, 1)
me.DescFrame:SetPoint( "TOPLEFT", me.NameFrame, "BOTTOMLEFT", 0, -10 )
me.DescFrame:SetHeight( 60 )
me.DescFrame:SetPoint( "RIGHT", me.NameFrame, "RIGHT" )

me.DescFrameText = me.DescFrame:CreateFontString( nil , "OVERLAY", "ChatFontNormal")
me.DescFrameText:SetPoint( "TOPLEFT", me.DescFrame, "TOPLEFT", 8, -8 )
me.DescFrameText:SetPoint( "BOTTOM", me.DescFrame, "BOTTOM" )
me.DescFrameText:SetWidth( 40 )
me.DescFrameText:SetJustifyH( "LEFT" )
me.DescFrameText:SetJustifyV( "TOP" )
me.DescFrameText:SetText("Desc:")

me.DescFrameEBox = CreateFrame( "EditBox", "SMDescFrameEditbox", me.DescFrame ) --, "InputBoxTemplate" )
local EBox = me.DescFrameEBox
EBox:SetFontObject("ChatFontNormal")
EBox:SetTextInsets(5, 5, 3, 3)
EBox:SetPoint("TOPLEFT", me.DescFrameText, "TOPRIGHT", 0 , 3 )
EBox:SetPoint("BOTTOMRIGHT", me.DescFrame, "BOTTOM", 5, 5 )
EBox:SetPoint("RIGHT", me.DescFrame, "RIGHT", -5, 0)
EBox:SetMultiLine(true)
EBox:SetBackdrop({
        bgFile="",
        edgeFile="Interface\\Tooltips\\UI-Tooltip-Border",
        tile="true",
        tileSize= 32,
        edgeSize=10,
        insets = {left=5, right=5, top=5, bottom=5}
    })
EBox:SetAutoFocus(false)
EBox:SetMaxLetters( 512 )
--EBox:SetScript("OnEnterPressed" , function(self)
--	self:ClearFocus()
--end)
EBox:SetScript("OnEditFocusGained", function(self)
	self:HighlightText()
end)
EBox:SetScript("OnEscapePressed", function(self)
	self:SetCursorPosition( 0 )
	self:ClearFocus()
end)

EBox:SetScript("OnEnter", function(self)
	me.HintFrameText:SetText( "Set the description of your command here" )
end)
EBox:SetScript("OnLeave", function(self)
	me.HintFrameText:SetText( " " )
end)

me.CodeFrame = CreateFrame("Frame" , "SMDescFrame", me )
me.CodeFrame:SetBackdrop( Backdrop2 )
me.CodeFrame:SetBackdropColor( 128, 128,128, 1)
me.CodeFrame:SetBackdropBorderColor( 255, 255, 255, 1)
me.CodeFrame:SetPoint( "TOPLEFT", me.DescFrame, "BOTTOMLEFT", 0, -10 )
me.CodeFrame:SetPoint( "BOTTOM", me.ButtonFrame, "BOTTOM" )
me.CodeFrame:SetPoint( "RIGHT", me.DescFrame, "RIGHT") 

me.CodeFrameText = me.DescFrame:CreateFontString( nil , "OVERLAY", "ChatFontNormal")
me.CodeFrameText:SetPoint( "TOPLEFT", me.CodeFrame, "TOPLEFT", 8, -8 )
me.CodeFrameText:SetPoint( "BOTTOM", me.CodeFrame, "BOTTOM" )
me.CodeFrameText:SetWidth( 40 )
me.CodeFrameText:SetJustifyH( "LEFT" )
me.CodeFrameText:SetJustifyV( "TOP" )
me.CodeFrameText:SetText("Code:")

me.CodeFrameEBox = CreateFrame( "EditBox", "SMCodeFrameEditbox", me.CodeFrame ) --, "InputBoxTemplate" )
local EBox = me.CodeFrameEBox
EBox:SetFontObject("ChatFontNormal")
EBox:SetTextInsets(5, 5, 3, 3)
EBox:SetPoint("TOPLEFT", me.CodeFrameText, "TOPRIGHT", 0 , 3 )
EBox:SetPoint("BOTTOMRIGHT", me.CodeFrame, "BOTTOM", 5, 5 )
EBox:SetPoint("RIGHT", me.CodeFrame, "RIGHT", -5, 0)
EBox:SetMultiLine(true)
EBox:SetFont("Interface\\addons\\SlashMagic\\VeraMono.ttf", 12 )
EBox:SetBackdrop({
        bgFile="",
        edgeFile="Interface\\Tooltips\\UI-Tooltip-Border",
        tile="true",
        tileSize= 32,
        edgeSize=10,
        insets = {left=5, right=5, top=5, bottom=5}
    })
EBox:SetAutoFocus(false)
EBox:SetMaxLetters( 1024 )
EBox:SetScript("OnEditFocusGained", function(self)
	self:HighlightText()
end)
EBox:SetScript("OnEscapePressed", function(self)
	self:SetCursorPosition( 0 )
	self:ClearFocus()
end)
EBox:SetScript("OnEnter", function(self)
	me.HintFrameText:SetText( "Edit the code for your slash command here. Your function will be called as: function(msg)  Using \"--\" anywhere on a line will cause the rest of the line to be a comment even if in quotes." )
end)
EBox:SetScript("OnLeave", function(self)
	me.HintFrameText:SetText( " " )
end)

me.HintFrame = CreateFrame("Frame" , "SMHintFrame", me )
me.HintFrame:SetBackdrop( Backdrop2 )
me.HintFrame:SetBackdropColor( 128, 128,128, 1)
me.HintFrame:SetBackdropBorderColor( 255, 255, 255, 1)
me.HintFrame:SetPoint( "TOPLEFT", me.ButtonFrame, "BOTTOMLEFT", 0, -10 )
me.HintFrame:SetPoint( "BOTTOM", me.Close, "TOP", 0, 10 )
me.HintFrame:SetPoint( "RIGHT", me, "RIGHT", -10, 0) 

me.HintFrameText = me.DescFrame:CreateFontString( nil , "OVERLAY", "ChatFontNormal")
me.HintFrameText:SetAllPoints( me.HintFrame )
me.HintFrameText:SetJustifyH( "CENTER" )
me.HintFrameText:SetJustifyV( "MIDDLE" )
me.HintFrameText:SetText(" ")


-- ****************************************************************************
-- * New Command Entry Frame                                                  *
-- ****************************************************************************
me.NewCmdFrame = CreateFrame("Frame", "SMNewCmdFrame", me )
local NewCmdFrame = me.NewCmdFrame
NewCmdFrame:SetFrameStrata("TOOLTIP")
NewCmdFrame:SetBackdrop( Backdrop )
NewCmdFrame:SetBackdropColor( 128, 128,128, 1)
NewCmdFrame:SetBackdropBorderColor( 255, 255, 255, 1)
NewCmdFrame:SetHeight( 240 )
NewCmdFrame:SetWidth( 480 )
NewCmdFrame:SetPoint( "CENTER", UIParent, "CENTER" )
NewCmdFrame:SetMovable(true)
NewCmdFrame:SetClampedToScreen(true)
NewCmdFrame:EnableMouse(true)
NewCmdFrame:Hide()

NewCmdFrame:SetScript("OnShow", function(self)
	me.FrameText:SetText("Enter the name of your new command.")
end)

-- Title region
local TitleRegion = NewCmdFrame:CreateTitleRegion()
TitleRegion:SetSize( 324, 64)
TitleRegion:SetPoint( "TOP", NewCmdFrame, "TOP", 0,12)

-- Title frame header
local TitleBkgnd= NewCmdFrame:CreateTexture(nil , "ARTWORK")
TitleBkgnd:SetTexture("Interface\\DialogFrame\\UI-DialogBox-Header")
TitleBkgnd:SetSize( 324, 64)
TitleBkgnd:SetPoint( "TOP", NewCmdFrame, "TOP", 0,12)

-- Title frame text
local Title = NewCmdFrame:CreateFontString( nil , "OVERLAY", "GameFontHighlight")
Title:SetText("Slash Magic")
Title:SetPoint( "CENTER", TitleBkgnd, "CENTER", 0, 12 )
Title:SetJustifyH( "CENTER" )
Title:SetJustifyV( "MIDDLE" )

-- Message Box text
me.FrameText= NewCmdFrame:CreateFontString( nil , "OVERLAY", "ChatFontNormal")
local FrameText = me.FrameText
FrameText:SetPoint( "TOPLEFT", NewCmdFrame, "TopLeft", 20, -20 )
FrameText:SetPoint( "BOTTOMRIGHT", NewCmdFrame, "BOTTOMRIGHT", -20, 100 )

me.EBox = CreateFrame( "EditBox", "SMNewCmdEditbox", NewCmdFrame, "InputBoxTemplate" )
local EBox = me.EBox
EBox:SetFontObject("ChatFontNormal")
EBox:SetTextInsets(0, 0, 3, 3)
EBox:SetPoint("TOPLEFT", FrameText , "BOTTOMLEFT", 0 ,-15 )
EBox:SetPoint("RIGHT", FrameText, "RIGHT" )
EBox:SetHeight(19)
--EBox:SetWidth(50)
EBox:SetAutoFocus(true)
EBox:SetMaxLetters( 254 )
EBox:SetScript("OnEnterPressed" , function(self)
	self:ClearFocus()
	me.CreateButton:Click("LeftButton")
end)
EBox:SetScript("OnEditFocusGained", function(self)
	self:HighlightText()
end)
EBox:SetScript("OnEscapePressed", function(self)
	self:SetCursorPosition( 0 )
	self:ClearFocus()
end)

-- Create Button
me.CreateButton = CreateFrame( "Button", "SMNewCmdFrameCreateButton", NewCmdFrame, "UIPanelButtonTemplate" )
local CreateButton = me.CreateButton
CreateButton:SetPoint("CENTER", NewCmdFrame, "BOTTOM", -85, 25)
CreateButton:SetSize( 150, 25)
CreateButton:SetText("CREATE")
CreateButton:SetScript("OnClick", function(self, button, down)
	if button ~= "LeftButton" then
		return
	end
	PlaySound( "GAMEGENERICBUTTONPRESS" )
	local NewCmd = string.lower(EBox:GetText())
	if type(SlashMagic_Commands.cmds[NewCmd]) == "boolean" then
		FrameText:SetText( ("The command |cFFFF0000[ %s ]|r already exists as part of this addon. Please edit that command or choose a different name."):format(NewCmd) )
		me.EBox:SetFocus()
		return
	end
	if SlashCmdList[ string.upper(NewCmd)] ~= nil then
		FrameText:SetText( ("The command |cFFFF0000[ %s ]|r already exists as part of some other addon. Please choose a different name."):format(NewCmd) )
		me.EBox:SetFocus()
		return
	end
	SlashMagic_Commands.cmds[NewCmd] = false
	SlashMagic_Commands.code[NewCmd] = { "return" ,}
	SlashMagic_Commands.desc[NewCmd] = "Enter a description here"
	Selected = NewCmd
	CmdListDisplayUpdate()
	EBox:SetText("")
	NewCmdFrame:Hide()
end)

local CancelButton = CreateFrame( "Button", "SMNewCmdFrameCancelButton", NewCmdFrame, "UIPanelButtonTemplate" )
CancelButton:SetPoint("CENTER", NewCmdFrame, "BOTTOM", 85, 25)
CancelButton:SetSize( 150, 25)
CancelButton:SetText("CANCEL")
CancelButton:SetScript("OnClick", function(self, button, down)
	if button ~= "LeftButton" then
		return
	end
	PlaySound( "GAMEGENERICBUTTONPRESS" )
	NewCmdFrame:Hide()
end)


